@@ -1,4 +1,9 @@
-## Copyright (C) 2014-2019 Pantxo Diribarne
+########################################################################
+##
+## Copyright (C) 2014-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {@var{retval} =} genpropdoc (@var{OBJNAME}, @var{FILENAME})
@@ -30,11 +37,11 @@
 ## @seealso{}
 ## @end deftypefn
 
-function genpropdoc (objname, fname)
-  objnames = {"root", "figure", "axes", ...
+function genpropdoc (objname, fname = "", props = {})
+  objnames = {"root", "figure", "axes", "legend", ...
               "image", "light", "line", "patch", "surface", "text", ...
               "uibuttongroup", "uicontextmenu", "uicontrol", "uipanel", ...
-              "uimenu", "uipushtool", "uitable", "uitoggletool" "uitoolbar"
+              "uimenu", "uipushtool", "uitable", "uitoggletool", "uitoolbar"
              };
 
   ## Base properties
@@ -42,15 +49,15 @@ function genpropdoc (objname, fname)
 
   ## Object properties
   if (any (strcmp (objname, objnames)))
-    obj = getstructure (objname, base);
+    obj = getstructure (objname, base, props);
   else
     error ("genpropdoc: unknown object %s", objname);
   endif
 
   ## Docstring
-  str = printdoc (objname, obj);
+  str = printdoc (objname, obj, ! isempty (props));
 
-  if (nargin == 2)
+  if (! isempty (fname))
     fid = fopen (fname,  "w+");
     if (fid < 0)
       error ("genpropdoc: couldn't open %s.", fname);
@@ -117,7 +124,11 @@ __prop__ is unused.";
   doc_fontname = "Name of font used for text rendering.  When setting \
 this property, the text rendering engine will search for a matching \
 font in your system.  If none is found then text is rendered using a \
-default sans serif font (same as the default @qcode{\"*\"} value).";
+default sans serif font (same as the default @qcode{\"*\"} value).\n\n\
+Programming Note: On systems that don’t use FontConfig natively \
+(all but Linux), the font cache is built when Octave is installed.  \
+You will need to run @code{system (\"fc-cache -fv\")} manually after \
+installing new fonts.";
   doc_fontunits = "Units used to interpret the @qcode{\"fontsize\"} property.";
   doc_fontweight = "Control the variant of the base font used for \
 text rendering.";
@@ -498,16 +509,28 @@ with the @code{paperorientation} property.";
 
       case "paperunits"
         s.doc = "The unit used to compute the @code{paperposition} property.  \
-For __prop__ set to @code{\"pixels\"}, the conversion between physical \
-units (ex: @code{\"inches\"}) and @code{\"pixels\"} is dependent on the \
-@code{screenpixelsperinch} property of the root object.";
+The conversion from physical units (e.g., @code{\"inches\"}) is dependent on \
+the @code{screenpixelsperinch} property of the root object.";
 
       case "pointer"
-        s.doc = doc_unused;
+        s.doc = "Name of the mouse pointer shape associated with the canvas \
+of the figure.  When __prop__ is \"custom\", the shape is determined by \
+the @code{pointershapecdata} property.\n\n\
+__prop__ has no effect when the figure is in zoom, pan, or rotate mode. \
+In this case, Octave automatically uses a pointer shape appropriate \
+to the mode.";
+
       case "pointershapecdata"
-        s.doc = doc_unused;
+        s.doc ="m-by-m matrix defining a custom pointer.  Each \
+element defines a pixel with the element (1,1) representing the \
+top-left pixel.  A value of 1 is colored black, a value of 2 is colored white, \
+and all other values are rendered as transparent.";
+        s.valid = "16-by-16 or 32-by-32 Matrix";
+
       case "pointershapehotspot"
-        s.doc = doc_unused;
+        s.doc ="For custom pointers only __prop__ defines the row and column \
+of the pixel in @code{pointershapecdata} that is used as the pointer location.";
+        s.valid = valid_2elvec;
 
       case "position"
         s.doc = "Specify the position and size of the figure canvas.  \
@@ -536,7 +559,19 @@ changing its @qcode{\"position\"} property.";
         s.valid = valid_fcn;
 
       case "selectiontype"
-        ## FIXME: docstring explaining what "{normal}|open|alt|extend" mean.
+        s.doc = "Selection type of the latest mouse click.\n\n\
+__prop__ may take different values depending on the combination of mouse \
+button and keyboard modifier that were used:\n\
+@table @code\n\
+@item normal:\n\
+Left-click.\n\
+@item alt:\n\
+Right-click or Ctrl+Left-click.\n\
+@item extend:\n\
+Shitf+Left-click, Middle click, or combined Left-click and Right-click.\n\
+@item open:\n\
+Double Left-click.\n\
+@end table";
 
       case "sizechangedfcn"
         s.doc = "Callback triggered when the figure window size is changed.\
@@ -581,7 +616,21 @@ the figure has focus.\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "windowscrollwheelfcn"
-        s.doc = doc_unused;
+        s.doc = "Function that is executed when a user manipulates \
+the mouse wheel over this figure.  \
+The function is called with two input arguments.  The first \
+argument holds the handle of the calling figure.  The second argument holds \
+an event structure which has the following members:\n\
+@table @code\n\
+@item VerticalScrollCount:\n\
+The number of wheel steps, typically 1 when scrolling down and -1 when \
+scrolling up.\n\
+@item VerticalScrollAmount:\n\
+The number of lines a wheel step should scroll.  This value is always 3.\n\
+@item EventName:\n\
+The event name which is \"WindowScrollWheel\".\n\
+@end table\
+\n\n__fcnmsg__";
         s.valid = valid_fcn;
 
       case "windowstyle"
@@ -600,7 +649,7 @@ Changing modes of a visible figure may cause the figure to close and reopen.";
     endswitch
 
   ## Axes properties
-  elseif (strcmp (objname, "axes"))
+  elseif (strcmp (objname, "axes") || strcmp (objname, "legend"))
     switch (field)
       ## Overridden shared properties
       case "clipping"
@@ -621,7 +670,7 @@ and @ref{XREFaxesposition, , @w{outerposition property}}.";
         s.doc = doc_unused;
 
       case "box"
-        s.doc = "Control whether the axes has a surrounding box.";
+        s.doc = "Control whether the __objname__ has a surrounding box.";
 
       case "boxstyle"
         s.doc = "For 3-D axes, control whether the @qcode{\"full\"} \
@@ -654,7 +703,8 @@ __modemsg__.  @xref{XREFpcolor, , @w{pcolor function}}.";
         s.doc = doc_unused;
 
       case "color"
-        s.doc = "Color of the axes background.  @xref{Colors, , colorspec}.";
+        s.doc = "Color of the __objname__ background.  \
+@xref{Colors, , colorspec}.";
         s.valid = valid_color;
 
       case "colororder"
@@ -696,12 +746,12 @@ to be the same as the length of 2 units on the y-axis.  \
         s.doc = doc_fontsize;
         s.valid = "scalar";
 
+      case "fontsmoothing"
+        s.doc = "Control whether any text associated with __objname__ is anti-aliased.";
+
       case "fontunits"
         s.doc = doc_fontunits;
 
-      case "fontsmoothing"
-        s.doc = doc_unused;
-
       case "fontweight"
         s.doc = doc_fontweight;
 
@@ -770,14 +820,20 @@ left corner of the axes at @math{(0.2, 0.3)} and the width and \
 height to be 0.4 and 0.5 respectively.  \
 @xref{XREFaxesposition, , @w{position property}}.";
         s.valid = valid_4elvec;
-
       case "plotboxaspectratio"
         s.doc = "@xref{XREFpbaspect, , pbaspect function}.  \
 __modemsg__.";
 
       case "plotboxaspectratiomode"
       case "position"
-        s.doc = "Specify the position of the plot excluding titles, \
+        if (strcmp (objname, "legend"))
+          s.doc = "Specify the position of the legend excluding its title. \
+The four elements of the vector are the coordinates of the lower left corner \
+and width and height of the legend.  Changing this property also \
+switches the @qcode{\"location\"} to @qcode{\"none\"}.";
+          s.printdefault = false;
+        else
+          s.doc = "Specify the position of the plot excluding titles, \
 axes, and legend.  The four elements of the vector are the \
 coordinates of the lower left corner and width and height of the \
 plot, in units normalized to the width and height of the plot \
@@ -785,6 +841,7 @@ window.  For example, @code{[0.2, 0.3, 0.4, 0.5]} sets the lower \
 left corner of the axes at @math{(0.2, 0.3)} and the width and \
 height to be 0.4 and 0.5 respectively.  \
 @xref{XREFaxesouterposition, , @w{outerposition property}}.";
+        endif
         s.valid = valid_4elvec;
 
       case "projection"
@@ -827,8 +884,13 @@ label fontsize";
         s.doc = "Control variant of base font used for the axes title.";
 
       case "units"
-        s.doc = "Units used to interpret the @qcode{\"position\"}, \
+        if (strcmp (objname, "legend"))
+          s.doc = "Units used to interpret the @qcode{\"position\"}, \
+ property.";
+        else
+          s.doc = "Units used to interpret the @qcode{\"position\"}, \
 @qcode{\"outerposition\"}, and @qcode{\"tightinset\"} properties.";
+        endif
 
       case "view"
         s.doc = "Two-element vector @code{[azimuth elevation]} specifying \
@@ -964,6 +1026,82 @@ for the z-axis.  __modemsg__.  @xref{XREFzlim, , @w{zlim function}}.";
         s.doc = doc_unused;
 
       case "ztickmode"
+
+      ## Legend specific properties
+      case "autoupdate"
+        s.doc = "Control whether the number of legend items is updated \
+automatically when objects are added to (or deleted from) the peer axes.\n\
+For example:\n\
+@example\n\
+@group\n\
+## Create a single plot with its legend.\n\
+figure ();\n\
+plot (1:10);\n\
+legend (\"Slope 1\");\n\
+## Add another plot and specify its displayname so that\n\
+## the legend is correctly updated.\n\
+hold on;\n\
+plot ((1:10) * 2, \"displayname\", \"Slope 2\");\n\
+## Stop automatic updates for further plots.\n\
+legend (\"autoupdate\", \"off\");\n\
+plot ((1:10) * 3);\n\
+@end group\n\
+@end example";
+
+      case "edgecolor"
+        s.doc = "Control the color of the legend outline.";
+        s.valid = valid_color;
+
+      case "interpreter"
+        s.doc = "Control if and eventually how labels strings are interpreted \
+before rendering.\n\
+@xref{XREFinterpreterusage, , @w{Use of the interpreter property}}.";
+
+      case "location"
+        s.doc = "Control the location of the legend.";
+
+      case "numcolumns"
+        s.doc = "Control the number of columns used in the layout of the legend items. \
+ For example:\n\
+@example\n\
+@group\n\
+figure ();\n\
+plot (rand (30));\n\
+legend (\"numcolumns\", 3);\n\
+@end group\n\
+@end example\n\
+__modemsg__.";
+        s.valid = "scalar interger";
+
+      case "orientation"
+        s.doc = "Control whether the legend items are arranged vertically \
+(column-wise) or horizontally (row-wise).";
+
+      case "string"
+        s.doc = "List of labels for the legend items.  For example:\n\
+@example\n\
+@group\n\
+figure ();\n\
+plot (rand (20));\n\
+## Let legend choose names automatically\n\
+hl = legend ();\n\
+## Selectively change some names\n\
+str = get (hl, \"string\");\n\
+str(1:5:end) = \"Garbage\";\n\
+set (hl, \"string\", str);\n\
+@end group\n\
+@end example";
+        s.valid = valid_cellstring;
+        s.printdefault = false;
+
+      case "textcolor"
+        s.doc = "Control the color of the text strings for legend items.";
+        s.valid = valid_color;
+
+      case "textposition"
+        s.doc = "Control whether text strings are displayed on the left or \
+right of their corresponding icon.";
+
     endswitch
 
   ## Line properties
@@ -1082,6 +1220,9 @@ and location of the text string.";
         s.doc = doc_fontname;
         s.valid = valid_string;
 
+      case "fontsmoothing"
+        s.doc = "Control whether anti-aliasing is used when rendering text.";
+
       case "fontsize"
         s.doc = doc_fontsize;
         s.valid = "scalar";
@@ -1641,6 +1782,11 @@ point source (@qcode{\"local\"}).";
       case "cdata"
       case "enable"
       case "extent"
+        s.doc = "Size of the text string associated to the uicontrol \
+ returned in the form @code{[0 0 width height]} (the two first elements \
+are always zero).\n\n\
+For multi-line strings the returned @code{width} and @code{height} \
+indicate the size of the rectangle enclosing all lines.";
         s.valid = valid_4elvec;
         s.printdefault = false;
 
@@ -1791,7 +1937,7 @@ functions see @ref{Callbacks, , @w{Callbacks section}}.";
   strout = strrep (strout, "__fcnmsg__", fcnmsg);
 endfunction
 
-function s = getstructure (objname, base = [])
+function s = getstructure (objname, base = [], props = {})
   hf = [];
   if (! strcmp (objname, "root"))
     ## Use an improbable number to avoid ishghandle to return true for 1
@@ -1807,6 +1953,15 @@ function s = getstructure (objname, base = [])
     h = 0;
   elseif (strcmp (objname, "figure"))
     h = hf;
+  elseif (strcmp (objname, "legend"))
+    line ();
+    h = legend ();
+    if (isempty (props))
+        props = {"autoupdate", "box", "color", "edgecolor", "fontangle", ...
+                 "fontname", "fontsize", "fontunits", "fontweight", ...
+                 "location", "numcolumns", "orientation", "position", ...
+                 "string", "textcolor", "title", "units"};
+    endif
   else
     eval (["h = " objname " ();"]);
   endif
@@ -1814,6 +1969,15 @@ function s = getstructure (objname, base = [])
   gprop = get (h);
   sprop = set (h);
 
+  if (! isempty (props))
+    flds = fieldnames (gprop);
+    idx = cellfun (@(s) ! any (strcmp (props, s)), flds);
+    gprop = rmfield (gprop, flds(idx));
+    flds = fieldnames (sprop);
+    idx = cellfun (@(s) ! any (strcmp (props, s)), flds);
+    sprop = rmfield (sprop, flds(idx));
+  endif
+
   fields = fieldnames (gprop);
   nf = numel (fields);
   args = cell (2*nf, 1);
@@ -1901,7 +2065,7 @@ function def = getdefault (h, objname, field)
 
 endfunction
 
-function str = printdoc (objname, obj)
+function str = printdoc (objname, obj, is_prop_subset)
   ## Sort fields so that they appear in alphabetic order in the manual
   fields = sort (fieldnames (obj));
   nf = numel (fields);
@@ -1910,6 +2074,8 @@ function str = printdoc (objname, obj)
   str = warn_autogen ();
   if (strcmp (objname, "root"))
     str = sprintf ("%s\n\nProperties of the root graphics object:", str);
+  elseif (is_prop_subset)
+    ## Do nothing
   else
     str = sprintf ("%s\n\nProperties of @code{%s} objects (@pxref{XREF%s,,%s}):",
                    str, objname, objname, objname);
@@ -1920,10 +2086,12 @@ function str = printdoc (objname, obj)
     field = fields{ii};
     str = sprintf ("%s\n\n", str);
 
-    ## @anchor: cross reference using XREFobjnamefield label
-    ## Concept index: call info from octave with 'doc ("objname field")'
-    str = sprintf ("%s@anchor{XREF%s%s}\n@prindex %s %s\n",
-                   str, objname, field, objname, field);
+    if (! is_prop_subset)
+      ## @anchor: cross reference using XREFobjnamefield label
+      ## Concept index: call info from octave with 'doc ("objname field")'
+      str = sprintf ("%s@anchor{XREF%s%s}\n@prindex %s %s\n",
+                     str, objname, field, objname, field);
+    endif
 
     ## Item
     str = sprintf ("%s@item @code{%s}", str, field);
