@@ -2051,8 +2051,13 @@ void TerminalView::mouseDoubleClickEvent(QMouseEvent* ev)
 
 void TerminalView::wheelEvent( QWheelEvent* ev )
 {
+#if defined (HAVE_QWHEELEVENT_ANGLEDELTA)
+  if (ev->angleDelta().y() == 0)
+    return;
+#else
   if (ev->orientation() != Qt::Vertical)
     return;
+#endif
 
   if ( _mouseMarks )
     _scrollBar->event(ev);
@@ -2060,9 +2065,19 @@ void TerminalView::wheelEvent( QWheelEvent* ev )
     {
       int charLine;
       int charColumn;
-      getCharacterPosition( ev->pos() , charLine , charColumn );
+#if defined (HAVE_QWHEELEVENT_POSITION)
+      QPoint pos = ev->position().toPoint();
+#else
+      QPoint pos = ev->pos();
+#endif
+      getCharacterPosition( pos , charLine , charColumn );
 
-      emit mouseSignal( ev->delta() > 0 ? 4 : 5,
+#if defined (HAVE_QWHEELEVENT_ANGLEDELTA)
+      int delta = ev->angleDelta().y();
+#else
+      int delta = ev->delta();
+#endif
+      emit mouseSignal( delta > 0 ? 4 : 5,
                         charColumn + 1,
                         charLine + 1 +_scrollBar->value() -_scrollBar->maximum() ,
                         0);
