@@ -27,6 +27,7 @@
 #  include "config.h"
 #endif
 
+#include <algorithm>
 #include <limits>
 
 #include <QApplication>
@@ -53,6 +54,7 @@
 #include "gui-preferences-global.h"
 #include "gui-preferences-ve.h"
 #include "octave-qobject.h"
+#include "qt-utils.h"
 #include "shortcut-manager.h"
 #include "variable-editor-model.h"
 #include "variable-editor.h"
@@ -526,7 +528,8 @@ namespace octave
         // font, so any character will do.  If not, you lose!
 
         QFontMetrics fm (font ());
-        int w = m_var_model->column_width () * fm.width ('0');
+        int w = (m_var_model->column_width ()
+                 * qt_fontmetrics_horizontal_advance (fm, '0'));
         horizontalHeader ()->setDefaultSectionSize (w);
       }
   }
@@ -838,7 +841,7 @@ namespace octave
 
     QItemSelectionModel *sel = selectionModel ();
     QList<QModelIndex> indices = sel->selectedIndexes ();
-    qSort (indices);
+    std::sort (indices.begin (), indices.end ());
 
     if (indices.isEmpty ())
       return;
