@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 1999-2019 John W. Eaton
-Copyright (C) 2009-2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1999-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -61,6 +63,23 @@ Cell::Cell (const string_vector& sv, bool trim)
     }
 }
 
+Cell::Cell (const std::list<std::string>& sl)
+  : Array<octave_value> ()
+{
+  octave_idx_type n = sl.size ();
+
+  if (n > 0)
+    {
+      resize (dim_vector (n, 1));
+
+      octave_value *dst = fortran_vec ();
+      auto p = sl.begin ();
+
+      for (octave_idx_type i = 0; i < n; i++)
+        dst[i] = *p++;
+    }
+}
+
 Cell::Cell (const Array<std::string>& sa)
   : Array<octave_value> (sa.dims ())
 {
@@ -156,7 +175,7 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
   octave_idx_type n = idx_arg.length ();
 
   // If we catch an indexing error in index_vector, we flag an error
-  // in index k.  Ensure it is the right value befor each idx_vector
+  // in index k.  Ensure it is the right value before each idx_vector
   // call.  Same variable as used in for loop in default case.
 
   octave_idx_type k = 0;
@@ -166,6 +185,7 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
       switch (n)
         {
         case 0:
+          warn_empty_index ("cell array");
           retval = *this;
           break;
 
@@ -211,9 +231,11 @@ Cell::index (const octave_value_list& idx_arg, bool resize_ok) const
 }
 
 /*
-%!test
+%% This behavior is required for Matlab compatibility.
+%!shared a
 %! a = {"foo", "bar"};
-%! assert (a(), a);
+%!assert (a(), a);
+%!error <invalid empty index expression> a{}
 */
 
 void
