@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 2016-2019 John W. Eaton
-Copyright (C) 2009-2019 Michael Goffioul
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2009-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -149,14 +151,9 @@ namespace octave
       if (! instance)
         {
           instance = new ft_manager ();
-
-          if (instance)
-            singleton_cleanup_list::add (cleanup_instance);
+          singleton_cleanup_list::add (cleanup_instance);
         }
 
-      if (! instance)
-        error ("unable to create ft_manager!");
-
       return retval;
     }
 
@@ -170,6 +167,13 @@ namespace octave
               : nullptr);
     }
 
+    static octave_map get_system_fonts (void)
+    {
+      return (instance_ok ()
+              ? instance->do_get_system_fonts ()
+              : octave_map ());
+    }
+
     static void font_destroyed (FT_Face face)
     {
       if (instance_ok ())
@@ -188,6 +192,90 @@ namespace octave
     // in class text_renderer.
     ft_cache cache;
 
+    static octave_map do_get_system_fonts (void)
+    {
+      static octave_map font_map;
+
+      if (font_map.isempty ())
+        {
+#if defined (HAVE_FONTCONFIG)
+          FcConfig *config = FcConfigGetCurrent();
+          FcPattern *pat = FcPatternCreate ();
+          FcObjectSet *os = FcObjectSetBuild (FC_FAMILY, FC_SLANT, FC_WEIGHT,
+                                              FC_CHARSET, nullptr);
+          FcFontSet *fs = FcFontList (config, pat, os);
+
+          if (fs->nfont > 0)
+            {
+              // Mark fonts that have at least all printable ASCII chars
+              FcCharSet *minimal_charset =  FcCharSetCreate ();
+              for (int i = 32; i < 127; i++)
+                FcCharSetAddChar (minimal_charset, static_cast<FcChar32> (i));
+
+              string_vector fields (4);
+              fields(0) = "family";
+              fields(1) = "angle";
+              fields(2) = "weight";
+              fields(3) = "suitable";
+
+              dim_vector dv (1, fs->nfont);
+              Cell families (dv);
+              Cell angles (dv);
+              Cell weights (dv);
+              Cell suitable (dv);
+
+              unsigned char *family;
+              int val;
+              for (int i = 0; fs && i < fs->nfont; i++)
+                {
+                  FcPattern *font = fs->fonts[i];
+                  if (FcPatternGetString (font, FC_FAMILY, 0, &family)
+                      == FcResultMatch)
+                    families(i) = std::string (reinterpret_cast<char*> (family));
+                  else
+                    families(i) = "unknown";
+
+                  if (FcPatternGetInteger (font, FC_SLANT, 0, &val)
+                      == FcResultMatch)
+                    angles(i) = (val == FC_SLANT_ITALIC
+                                 || val == FC_SLANT_OBLIQUE)
+                                ? "italic" : "normal";
+                  else
+                    angles(i) = "unknown";
+
+                  if (FcPatternGetInteger (font, FC_WEIGHT, 0, &val)
+                      == FcResultMatch)
+                    weights(i) = (val == FC_WEIGHT_BOLD
+                                  || val == FC_WEIGHT_DEMIBOLD)
+                                 ? "bold" : "normal";
+                  else
+                    weights(i) = "unknown";
+
+                  FcCharSet *cset;
+                  if (FcPatternGetCharSet (font, FC_CHARSET, 0, &cset)
+                      == FcResultMatch)
+                    suitable(i) = (FcCharSetIsSubset (minimal_charset, cset)
+                                   ? true : false);
+                  else
+                    suitable(i) = false;
+                }
+
+              font_map = octave_map (dv, fields);
+
+              font_map.assign ("family", families);
+              font_map.assign ("angle", angles);
+              font_map.assign ("weight", weights);
+              font_map.assign ("suitable", suitable);
+
+              if (fs)
+                FcFontSetDestroy (fs);
+            }
+#endif
+        }
+
+      return font_map;
+    }
+
     FT_Face do_get_font (const std::string& name, const std::string& weight,
                          const std::string& angle, double size)
     {
@@ -245,10 +333,6 @@ namespace octave
 
           if (weight == "bold")
             fc_weight = FC_WEIGHT_BOLD;
-          else if (weight == "light")
-            fc_weight = FC_WEIGHT_LIGHT;
-          else if (weight == "demi")
-            fc_weight = FC_WEIGHT_DEMIBOLD;
           else
             fc_weight = FC_WEIGHT_NORMAL;
 
@@ -316,8 +400,8 @@ namespace octave
               retval->generic.finalizer = ft_face_destroyed;
 
               // Insert loaded font into the cache.
-
-              cache[key] = retval;
+              if (FT_Reference_Face (retval) == 0)
+                cache[key] = retval;
             }
 #endif
         }
@@ -334,6 +418,7 @@ namespace octave
           cache.erase (*pkey);
           delete pkey;
           face->generic.data = nullptr;
+          FT_Done_Face (face);
         }
     }
 
@@ -378,7 +463,7 @@ namespace octave
         xoffset (0), line_yoffset (0), yoffset (0), mode (MODE_BBOX),
         color (dim_vector (1, 3), 0), m_do_strlist (false), m_strlist (),
         line_xoffset (0), m_ymin (0), m_ymax (0), m_deltax (0),
-        m_max_fontsize (0)
+        m_max_fontsize (0), m_antialias (true)
     { }
 
     // No copying!
@@ -422,9 +507,13 @@ namespace octave
     Matrix get_extent (const std::string& txt, double rotation,
                        const caseless_str& interpreter);
 
+    void set_anti_aliasing (bool val) { m_antialias = val; };
+
     void set_font (const std::string& name, const std::string& weight,
                    const std::string& angle, double size);
 
+    octave_map get_system_fonts (void);
+
     void set_color (const Matrix& c);
 
     void set_mode (int m);
@@ -547,17 +636,26 @@ namespace octave
     // Used for computing the distance between lines.
     double m_max_fontsize;
 
+    // Anti-aliasing.
+    bool m_antialias;
+
   };
 
   void
-  ft_text_renderer::set_font (const std::string& name, const std::string& weight,
+  ft_text_renderer::set_font (const std::string& name,
+                              const std::string& weight,
                               const std::string& angle, double size)
   {
     // FIXME: take "fontunits" into account
-
     font = ft_font (name, weight, angle, size, nullptr);
   }
 
+  octave_map
+  ft_text_renderer::get_system_fonts (void)
+  {
+    return ft_manager::get_system_fonts ();
+  }
+
   void
   ft_text_renderer::push_new_line (void)
   {
@@ -712,6 +810,16 @@ namespace octave
         break;
       }
   }
+  bool is_opaque (const FT_GlyphSlot &glyph, const int x, const int y)
+  {
+    // Borrowed from https://stackoverflow.com/questions/14800827/
+    //    indexing-pixels-in-a-monochrome-freetype-glyph-buffer
+    int pitch = std::abs (glyph->bitmap.pitch);
+    unsigned char *row = &glyph->bitmap.buffer[pitch * y];
+    char cvalue = row[x >> 3];
+
+    return ((cvalue & (128 >> (x & 7))) != 0);
+  }
 
   FT_UInt
   ft_text_renderer::process_character (FT_ULong code, FT_UInt previous)
@@ -754,7 +862,9 @@ namespace octave
             switch (mode)
               {
               case MODE_RENDER:
-                if (FT_Render_Glyph (face->glyph, FT_RENDER_MODE_NORMAL))
+                if (FT_Render_Glyph (face->glyph, (m_antialias
+                                                   ? FT_RENDER_MODE_NORMAL
+                                                   : FT_RENDER_MODE_MONO)))
                   {
                     glyph_index = 0;
                     warn_glyph_render (code);
@@ -786,7 +896,11 @@ namespace octave
                     for (int r = 0; static_cast<unsigned int> (r) < bitmap.rows; r++)
                       for (int c = 0; static_cast<unsigned int> (c) < bitmap.width; c++)
                         {
-                          unsigned char pix = bitmap.buffer[r*bitmap.width+c];
+                          unsigned char pix
+                            = (m_antialias
+                               ? bitmap.buffer[r*bitmap.width+c]
+                               : (is_opaque (face->glyph, c, r) ? 255 : 0));
+
                           if (x0+c < 0 || x0+c >= pixels.dim2 ()
                               || y0-r < 0 || y0-r >= pixels.dim3 ())
                             {
@@ -866,7 +980,7 @@ namespace octave
 
     m_strlist = std::list<text_renderer::string> ();
 
-    octave::unwind_protect frame;
+    unwind_protect frame;
     frame.protect_var (m_do_strlist);
     frame.protect_var (m_strlist);
     m_do_strlist = true;
@@ -908,7 +1022,7 @@ namespace octave
                 mblen = 1;
                 u32_c = 0xFFFD;
               }
-                
+
             n -= mblen;
 
             if (m_do_strlist && mode == MODE_RENDER)
@@ -987,16 +1101,16 @@ namespace octave
     int saved_line_yoffset = line_yoffset;
     int saved_yoffset = yoffset;
 
+    double sz = font.get_size ();
+
+    // Reducing font size by 70% produces decent results.
     set_font (font.get_name (), font.get_weight (), font.get_angle (),
-              font.get_size () - 2);
+              std::max (5.0, sz * 0.7));
 
     if (font.is_valid ())
       {
-        int h = font.get_face ()->size->metrics.height >> 6;
-
-        // Shifting the baseline by 2/3 the font height seems to produce
-        // decent result.
-        yoffset -= (h * 2) / 3;
+        // Shifting the baseline by 15% of the font size gives decent results.
+        yoffset -= std::ceil (sz * 0.15);
 
         if (mode == MODE_BBOX)
           update_line_bbox ();
@@ -1018,16 +1132,16 @@ namespace octave
     int saved_line_yoffset = line_yoffset;
     int saved_yoffset = yoffset;
 
+    double sz = font.get_size ();
+
+    // Reducing font size by 70% produces decent results.
     set_font (font.get_name (), font.get_weight (), font.get_angle (),
-              font.get_size () - 2);
+              std::max (5.0, sz * 0.7));
 
     if (saved_font.is_valid ())
       {
-        int s_asc = saved_font.get_face ()->size->metrics.ascender >> 6;
-
-        // Shifting the baseline by 2/3 base font ascender seems to produce
-        // decent result.
-        yoffset += (s_asc * 2) / 3;
+        // Shifting the baseline by 40% of the font size gives decent results.
+        yoffset += std::ceil (sz * 0.4);
 
         if (mode == MODE_BBOX)
           update_line_bbox ();
