@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2000-2017 Kai Habel
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2000-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 /*
   16. July 2000 - Kai Habel: first release
@@ -42,17 +45,21 @@ along with Octave; see the file COPYING.  If not, see
 #  include "config.h"
 #endif
 
-#include <iostream>
+#include <cstdio>
+
+#include <limits>
 #include <string>
 
+#include "Array.h"
+#include "dMatrix.h"
+#include "dRowVector.h"
 #include "oct-locbuf.h"
+#include "unwind-prot.h"
 
-#include "Cell.h"
 #include "defun-dld.h"
 #include "error.h"
 #include "errwarn.h"
 #include "ovl.h"
-#include "unwind-prot.h"
 
 #if defined (HAVE_QHULL)
 
@@ -68,6 +75,19 @@ close_fcn (FILE *f)
   std::fclose (f);
 }
 
+static void
+free_qhull_memory ()
+{
+  qh_freeqhull (! qh_ALL);
+
+  int curlong, totlong;
+  qh_memfreeshort (&curlong, &totlong);
+
+  if (curlong || totlong)
+    warning ("__delaunayn__: did not free %d bytes of long memory (%d pieces)",
+             totlong, curlong);
+}
+
 static bool
 octave_qhull_dims_ok (octave_idx_type dim, octave_idx_type n, const char *who)
 {
@@ -113,7 +133,7 @@ Undocumented internal function.
   // Default options
   std::string options;
   if (dim <= 3)
-    options = "Qt Qbb Qc Qz";
+    options = "Qt Qbb Qc";
   else
     options = "Qt Qbb Qc Qx";
 
@@ -121,15 +141,15 @@ Undocumented internal function.
     {
       if (args(1).is_string ())
         options = args(1).string_value ();
-      else if (args(1).is_empty ())
+      else if (args(1).isempty ())
         ;  // Use default options
-      else if (args(1).is_cellstr ())
+      else if (args(1).iscellstr ())
         {
           options = "";
           Array<std::string> tmp = args(1).cellstr_value ();
 
           for (octave_idx_type i = 0; i < tmp.numel (); i++)
-            options += tmp(i) + " ";
+            options += tmp(i) + ' ';
         }
       else
         error ("__delaunayn__: OPTIONS argument must be a string, cell array of strings, or empty");
@@ -163,6 +183,9 @@ Undocumented internal function.
 
       int exitcode = qh_new_qhull (dim, n, pt_array,
                                    ismalloc, flags, outfile, errfile);
+
+      frame.add_fcn (free_qhull_memory);
+
       if (exitcode)
         error ("__delaunayn__: qhull failed");
 
@@ -184,41 +207,28 @@ Undocumented internal function.
             error ("__delaunayn__: Qhull returned non-simplicial facets -- try delaunayn with different options");
         }
 
-      if (! exitcode)
+      Matrix simpl (nf, dim+1);
+
+      FORALLfacets
         {
-          Matrix simpl (nf, dim+1);
-
-          FORALLfacets
+          if (! facet->upperdelaunay)
             {
-              if (! facet->upperdelaunay)
+              octave_idx_type j = 0;
+
+              FOREACHvertex_ (facet->vertices)
                 {
-                  octave_idx_type j = 0;
-
-                  FOREACHvertex_ (facet->vertices)
-                    {
-                      simpl(i, j++) = 1 + qh_pointid(vertex->point);
-                    }
-                  i++;
+                  simpl(i, j++) = 1 + qh_pointid(vertex->point);
                 }
+              i++;
             }
-
-          retval(0) = simpl;
         }
 
-      // Free memory from Qhull
-      qh_freeqhull (! qh_ALL);
-
-      int curlong, totlong;
-      qh_memfreeshort (&curlong, &totlong);
-
-      if (curlong || totlong)
-        warning ("__delaunay__: did not free %d bytes of long memory (%d pieces)",
-                 totlong, curlong);
+      retval(0) = simpl;
     }
   else if (n == dim + 1)
     {
-      // one should check if nx points span a simplex
-      // I will look at this later.
+      // FIXME: One should check if nx points span a simplex.
+      //        I will look at this later.
       RowVector vec (n);
       for (octave_idx_type i = 0; i < n; i++)
         vec(i) = i + 1.0;
