@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2012-2019 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2012-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_tree_classdef_h)
 #define octave_tree_classdef_h 1
@@ -41,6 +44,94 @@ namespace octave
 {
   class interpreter;
 
+  class tree_superclass_ref : public tree_expression
+  {
+  public:
+
+    tree_superclass_ref (void) = delete;
+
+    tree_superclass_ref (const std::string& meth, const std::string& cls,
+                         int l = -1, int c = -1)
+      : tree_expression (l, c), m_method_name (meth), m_class_name (cls)
+    { }
+
+    // No copying!
+
+    tree_superclass_ref (const tree_superclass_ref&) = delete;
+
+    tree_superclass_ref& operator = (const tree_superclass_ref&) = delete;
+
+    std::string method_name (void) const
+    {
+      return m_method_name;
+    }
+
+    std::string class_name (void) const { return m_class_name; }
+
+    tree_superclass_ref * dup (symbol_scope& scope) const;
+
+    octave_value evaluate (tree_evaluator& tw, int nargout = 1)
+    {
+      octave_value_list retval = evaluate_n (tw, nargout);
+
+      return retval.length () > 0 ? retval(0) : octave_value ();
+    }
+
+    octave_value_list evaluate_n (tree_evaluator& tw, int nargout = 1);
+
+    void accept (tree_walker& tw)
+    {
+      tw.visit_superclass_ref (*this);
+    }
+
+  private:
+
+    // The name of the method to call.  This is the text before the
+    // "@" and may be of the form "object.method".
+    std::string m_method_name;
+
+    // The name of the superclass.  This is the text after the "@"
+    // and may be of the form "object.method".
+    std::string m_class_name;
+  };
+
+  class tree_metaclass_query : public tree_expression
+  {
+  public:
+
+    tree_metaclass_query (void) = delete;
+
+    tree_metaclass_query (const std::string& cls, int l = -1, int c = -1)
+      : tree_expression (l, c), m_class_name (cls)
+    { }
+
+    // No copying!
+
+    tree_metaclass_query (const tree_metaclass_query&) = delete;
+
+    tree_metaclass_query& operator = (const tree_metaclass_query&) = delete;
+
+    std::string class_name (void) const { return m_class_name; }
+
+    tree_metaclass_query * dup (symbol_scope& scope) const;
+
+    octave_value evaluate (tree_evaluator&, int nargout = 1);
+
+    octave_value_list evaluate_n (tree_evaluator& tw, int nargout = 1)
+    {
+      return ovl (evaluate (tw, nargout));
+    }
+
+    void accept (tree_walker& tw)
+    {
+      tw.visit_metaclass_query (*this);
+    }
+
+  private:
+
+    std::string m_class_name;
+  };
+
   class tree_classdef_attribute
   {
   public:
@@ -140,7 +231,8 @@ namespace octave
     std::string m_cls_name;
   };
 
-  class tree_classdef_superclass_list : public base_list<tree_classdef_superclass *>
+  class tree_classdef_superclass_list
+    : public base_list<tree_classdef_superclass *>
   {
   public:
 
@@ -176,9 +268,8 @@ namespace octave
   public:
 
     tree_classdef_element (tree_classdef_attribute_list *a,
-                           base_list<T> *elist,
-                           comment_list *lc, comment_list *tc,
-                           int l = -1, int c = -1)
+                           base_list<T> *elist, comment_list *lc,
+                           comment_list *tc, int l = -1, int c = -1)
       : tree (l, c), m_attr_list (a), m_elt_list (elist),
         m_lead_comm (lc), m_trail_comm (tc)
     { }
@@ -215,10 +306,10 @@ namespace octave
     // The list of objects contained in this block.
     base_list<T> *m_elt_list;
 
-    // Comment preceding the token marking the beginning of the block.
+    // Comments preceding the token marking the beginning of the block.
     comment_list *m_lead_comm;
 
-    // Comment preceding END token.
+    // Comments preceding the END token marking the end of the block.
     comment_list *m_trail_comm;
   };
 
@@ -226,10 +317,11 @@ namespace octave
   {
   public:
 
-    tree_classdef_property (tree_identifier *i = nullptr,
-                            tree_expression *e = nullptr)
-      : m_id (i), m_expr (e)
-    { }
+    tree_classdef_property (tree_identifier *i,
+                            comment_list *comments = nullptr);
+
+    tree_classdef_property (tree_identifier *i, tree_expression *e,
+                            comment_list *comments = nullptr);
 
     // No copying!
 
@@ -247,6 +339,14 @@ namespace octave
 
     tree_expression * expression (void) { return m_expr; }
 
+    comment_list * comments (void) const { return m_comments; }
+
+    void doc_string (const std::string& txt) { m_doc_string = txt; }
+
+    std::string doc_string (void) const { return m_doc_string; }
+
+    bool have_doc_string (void) const { return ! m_doc_string.empty (); }
+
     void accept (tree_walker& tw)
     {
       tw.visit_classdef_property (*this);
@@ -256,6 +356,8 @@ namespace octave
 
     tree_identifier *m_id;
     tree_expression *m_expr;
+    comment_list *m_comments;
+    std::string m_doc_string;
   };
 
   class tree_classdef_property_list : public base_list<tree_classdef_property *>
@@ -291,8 +393,7 @@ namespace octave
 
     tree_classdef_properties_block (tree_classdef_attribute_list *a,
                                     tree_classdef_property_list *plist,
-                                    comment_list *lc,
-                                    comment_list *tc,
+                                    comment_list *lc, comment_list *tc,
                                     int l = -1, int c = -1)
       : tree_classdef_element<tree_classdef_property *> (a, plist, lc, tc, l, c)
     { }
@@ -344,8 +445,8 @@ namespace octave
 
     tree_classdef_methods_block (tree_classdef_attribute_list *a,
                                  tree_classdef_methods_list *mlist,
-                                 comment_list *lc,
-                                 comment_list *tc, int l = -1, int c = -1)
+                                 comment_list *lc, comment_list *tc,
+                                 int l = -1, int c = -1)
       : tree_classdef_element<octave_value> (a, mlist, lc, tc, l, c)
     { }
 
@@ -368,7 +469,8 @@ namespace octave
   {
   public:
 
-    tree_classdef_event (tree_identifier *i = nullptr) : m_id (i) { }
+    tree_classdef_event (tree_identifier *i = nullptr,
+                         comment_list *comments = nullptr);
 
     // No copying!
 
@@ -383,6 +485,14 @@ namespace octave
 
     tree_identifier * ident (void) { return m_id; }
 
+    comment_list * comments (void) const { return m_comments; }
+
+    void doc_string (const std::string& txt) { m_doc_string = txt; }
+
+    std::string doc_string (void) const { return m_doc_string; }
+
+    bool have_doc_string (void) const { return ! m_doc_string.empty (); }
+
     void accept (tree_walker& tw)
     {
       tw.visit_classdef_event (*this);
@@ -391,6 +501,8 @@ namespace octave
   private:
 
     tree_identifier *m_id;
+    comment_list *m_comments;
+    std::string m_doc_string;
   };
 
   class tree_classdef_events_list : public base_list<tree_classdef_event *>
@@ -427,8 +539,8 @@ namespace octave
 
     tree_classdef_events_block (tree_classdef_attribute_list *a,
                                 tree_classdef_events_list *elist,
-                                comment_list *lc,
-                                comment_list *tc, int l = -1, int c = -1)
+                                comment_list *lc, comment_list *tc,
+                                int l = -1, int c = -1)
       : tree_classdef_element<tree_classdef_event *> (a, elist, lc, tc, l, c)
     { }
 
@@ -451,11 +563,8 @@ namespace octave
   {
   public:
 
-    tree_classdef_enum (void) : m_id (nullptr), m_expr (nullptr) { }
-
-    tree_classdef_enum (tree_identifier *i, tree_expression *e)
-      : m_id (i), m_expr (e)
-    { }
+    tree_classdef_enum (tree_identifier *i, tree_expression *e,
+                        comment_list *comments);
 
     // No copying!
 
@@ -473,6 +582,14 @@ namespace octave
 
     tree_expression * expression (void) { return m_expr; }
 
+    comment_list * comments (void) const { return m_comments; }
+
+    void doc_string (const std::string& txt) { m_doc_string = txt; }
+
+    std::string doc_string (void) const { return m_doc_string; }
+
+    bool have_doc_string (void) const { return ! m_doc_string.empty (); }
+
     void accept (tree_walker& tw)
     {
       tw.visit_classdef_enum (*this);
@@ -482,6 +599,8 @@ namespace octave
 
     tree_identifier *m_id;
     tree_expression *m_expr;
+    comment_list *m_comments;
+    std::string m_doc_string;
   };
 
   class tree_classdef_enum_list : public base_list<tree_classdef_enum *>
@@ -517,8 +636,8 @@ namespace octave
 
     tree_classdef_enum_block (tree_classdef_attribute_list *a,
                               tree_classdef_enum_list *elist,
-                              comment_list *lc,
-                              comment_list *tc, int l = -1, int c = -1)
+                              comment_list *lc, comment_list *tc,
+                              int l = -1, int c = -1)
       : tree_classdef_element<tree_classdef_enum *> (a, elist, lc, tc, l, c)
     { }
 
@@ -541,45 +660,33 @@ namespace octave
   {
   public:
 
-    typedef std::list<tree_classdef_properties_block *>::iterator properties_list_iterator;
-    typedef std::list<tree_classdef_properties_block *>::const_iterator properties_list_const_iterator;
+    typedef std::list<tree_classdef_properties_block *>::iterator
+      properties_list_iterator;
+    typedef std::list<tree_classdef_properties_block *>::const_iterator
+      properties_list_const_iterator;
 
-    typedef std::list<tree_classdef_methods_block *>::iterator methods_list_iterator;
-    typedef std::list<tree_classdef_methods_block *>::const_iterator methods_list_const_iterator;
+    typedef std::list<tree_classdef_methods_block *>::iterator
+      methods_list_iterator;
+    typedef std::list<tree_classdef_methods_block *>::const_iterator
+      methods_list_const_iterator;
 
     typedef std::list<tree_classdef_events_block *>::iterator events_list_iterator;
-    typedef std::list<tree_classdef_events_block *>::const_iterator events_list_const_iterator;
+    typedef std::list<tree_classdef_events_block *>::const_iterator
+      events_list_const_iterator;
 
     typedef std::list<tree_classdef_enum_block *>::iterator enum_list_iterator;
-    typedef std::list<tree_classdef_enum_block *>::const_iterator enum_list_const_iterator;
+    typedef std::list<tree_classdef_enum_block *>::const_iterator
+      enum_list_const_iterator;
 
-    tree_classdef_body (void)
-      : m_properties_lst (), m_methods_lst (), m_events_lst (), m_enum_lst ()
-    { }
+    tree_classdef_body (void);
 
-    tree_classdef_body (tree_classdef_properties_block *pb)
-      : m_properties_lst (), m_methods_lst (), m_events_lst (), m_enum_lst ()
-    {
-      append (pb);
-    }
+    tree_classdef_body (tree_classdef_properties_block *pb);
 
-    tree_classdef_body (tree_classdef_methods_block *mb)
-      : m_properties_lst (), m_methods_lst (), m_events_lst (), m_enum_lst ()
-    {
-      append (mb);
-    }
+    tree_classdef_body (tree_classdef_methods_block *mb);
 
-    tree_classdef_body (tree_classdef_events_block *evb)
-      : m_properties_lst (), m_methods_lst (), m_events_lst (), m_enum_lst ()
-    {
-      append (evb);
-    }
+    tree_classdef_body (tree_classdef_events_block *evb);
 
-    tree_classdef_body (tree_classdef_enum_block *enb)
-      : m_properties_lst (), m_methods_lst (), m_events_lst (), m_enum_lst ()
-    {
-      append (enb);
-    }
+    tree_classdef_body (tree_classdef_enum_block *enb);
 
     // No copying!
 
@@ -629,6 +736,12 @@ namespace octave
       return m_enum_lst;
     }
 
+    void doc_string (const std::string& txt) { m_doc_string = txt; }
+
+    std::string doc_string (void) const { return m_doc_string; }
+
+    bool have_doc_string (void) const { return ! m_doc_string.empty (); }
+
     void accept (tree_walker& tw)
     {
       tw.visit_classdef_body (*this);
@@ -636,6 +749,8 @@ namespace octave
 
   private:
 
+    std::string get_doc_string (comment_list *comment) const;
+
     std::list<tree_classdef_properties_block *> m_properties_lst;
 
     std::list<tree_classdef_methods_block *> m_methods_lst;
@@ -643,6 +758,8 @@ namespace octave
     std::list<tree_classdef_events_block *> m_events_lst;
 
     std::list<tree_classdef_enum_block *> m_enum_lst;
+
+    std::string m_doc_string;
   };
 
   // Classdef definition.
@@ -651,13 +768,13 @@ namespace octave
   {
   public:
 
-    tree_classdef (tree_classdef_attribute_list *a, tree_identifier *i,
+    tree_classdef (const octave::symbol_scope& scope,
+                   tree_classdef_attribute_list *a, tree_identifier *i,
                    tree_classdef_superclass_list *sc,
                    tree_classdef_body *b, comment_list *lc,
-                   comment_list *tc,
-                   const std::string& pn = "", int l = -1,
-                   int c = -1)
-      : tree_command (l, c), m_attr_list (a), m_id (i),
+                   comment_list *tc, const std::string& pn = "",
+                   int l = -1, int c = -1)
+      : tree_command (l, c), m_scope (scope), m_attr_list (a), m_id (i),
         m_supclass_list (sc), m_element_list (b), m_lead_comm (lc),
         m_trail_comm (tc), m_pack_name (pn)
     { }
@@ -678,6 +795,8 @@ namespace octave
       delete m_trail_comm;
     }
 
+    octave::symbol_scope scope (void) { return m_scope; }
+
     tree_classdef_attribute_list *
     attribute_list (void) { return m_attr_list; }
 
@@ -691,10 +810,15 @@ namespace octave
     comment_list * leading_comment (void) { return m_lead_comm; }
     comment_list * trailing_comment (void) { return m_trail_comm; }
 
-    const std::string& package_name (void) const { return m_pack_name; }
+    std::string package_name (void) const { return m_pack_name; }
 
-    octave_function * make_meta_class (interpreter& interp,
-                                       bool is_at_folder = false);
+    octave_value make_meta_class (interpreter& interp,
+                                  bool is_at_folder = false);
+
+    std::string doc_string (void) const
+    {
+      return m_element_list ? m_element_list->doc_string () : "";
+    }
 
     void accept (tree_walker& tw)
     {
@@ -703,6 +827,12 @@ namespace octave
 
   private:
 
+    // The scope that was used when parsing the classdef object and that
+    // corresponds to any identifiers that were found in attribute lists
+    // (for example).  Used again when computing the meta class object.
+
+    octave::symbol_scope m_scope;
+
     tree_classdef_attribute_list *m_attr_list;
 
     tree_identifier *m_id;
