@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 1995-2017 John W. Eaton
-Copyright (C) 2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1995-2020 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_chMatrix_h)
 #define octave_chMatrix_h 1
@@ -30,7 +32,6 @@ along with Octave; see the file COPYING.  If not, see
 
 #include "Array.h"
 #include "chNDArray.h"
-
 #include "mx-defs.h"
 #include "mx-op-decl.h"
 #include "str-vec.h"
@@ -43,7 +44,13 @@ charMatrix : public charNDArray
 
 public:
 
-  charMatrix (void) : charNDArray () { }
+  charMatrix (void) = default;
+
+  charMatrix (const charMatrix& a) = default;
+
+  charMatrix& operator = (const charMatrix& a) = default;
+
+  ~charMatrix (void) = default;
 
   charMatrix (octave_idx_type r, octave_idx_type c)
     : charNDArray (dim_vector (r, c)) { }
@@ -58,8 +65,6 @@ public:
 
   charMatrix (const Array<char>& a) : charNDArray (a.as_matrix ()) { }
 
-  charMatrix (const charMatrix& a) : charNDArray (a) { }
-
   charMatrix (char c) : charNDArray (c) { }
 
   charMatrix (const char *s) : charNDArray (s) { }
