@@ -1,4 +1,9 @@
-## Copyright (C) 2014-2019 Andreas Weber
+########################################################################
+##
+## Copyright (C) 2014-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn {} {@var{funcname} =} __get_funcname__ (@var{basename})
@@ -24,11 +31,9 @@
 ## __[basename]_[graphics_toolkit]__, use fltk as default.
 ## @end deftypefn
 
-## Author: Andreas Weber
-
 function funcname = __get_funcname__ (basename)
 
-  if (! __octave_link_enabled__ ())
+  if (! __event_manager_enabled__ ())
     tk = graphics_toolkit ();
     funcname = [ "__" basename "_" tk "__"];
     if (numel (tk) > 0 && ! strcmp (tk, "fltk")
