@@ -1,21 +1,27 @@
-## Copyright (C) 2005-2016 John W. Eaton
-## Copyright (C) 2013-2016 Arun Giridhar
+########################################################################
+##
+## Copyright (C) 2005-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{x}, @var{obj}, @var{info}, @var{iter}, @var{nf}, @var{lambda}] =} sqp (@var{x0}, @var{phi})
@@ -202,26 +208,29 @@ function [x, obj, info, iter, nf, lambda] = sqp (x0, objf, cef, cif, lb, ub, max
     x0 = x0';
   endif
 
+  have_grd = 0;
   have_hess = 0;
   if (iscell (objf))
     switch (numel (objf))
       case 1
         obj_fun = objf{1};
-        obj_grd = @(x) fd_obj_grd (x, obj_fun);
+        obj_grd = @(x, obj) fd_obj_grd (x, obj, obj_fun);
       case 2
         obj_fun = objf{1};
         obj_grd = objf{2};
+        have_grd = 1;
       case 3
         obj_fun = objf{1};
         obj_grd = objf{2};
         obj_hess = objf{3};
+        have_grd = 1;
         have_hess = 1;
       otherwise
         error ("sqp: invalid objective function specification");
     endswitch
   else
     obj_fun = objf;   # No cell array, only obj_fun set
-    obj_grd = @(x) fd_obj_grd (x, obj_fun);
+    obj_grd = @(x, obj) fd_obj_grd (x, obj, obj_fun);
   endif
 
   ce_fun = @empty_cf;
@@ -351,7 +360,11 @@ function [x, obj, info, iter, nf, lambda] = sqp (x0, objf, cef, cif, lb, ub, max
   obj = feval (obj_fun, x0);
   globals.nfun = 1;
 
-  c = feval (obj_grd, x0);
+  if (have_grd)
+    c = feval (obj_grd, x0);
+  else
+    c = feval (obj_grd, x0, obj);
+  endif
 
   ## Choose an initial NxN symmetric positive definite Hessian approximation B.
   n = length (x0);
@@ -410,7 +423,7 @@ function [x, obj, info, iter, nf, lambda] = sqp (x0, objf, cef, cif, lb, ub, max
 
     old_lambda = lambda;
     [p, obj_qp, INFO, lambda] = qp (x, B, c, F, g, [], [], d, C,
-                                    Inf (size (d)));
+                                    Inf (size (d)), struct ("TolX", tol));
 
     info = INFO.info;
 
@@ -433,10 +446,22 @@ function [x, obj, info, iter, nf, lambda] = sqp (x0, objf, cef, cif, lb, ub, max
     ## merit function phi.
     [x_new, alpha, obj_new, globals] = ...
         linesearch_L1 (x, p, obj_fun, obj_grd, ce_fun, ci_fun, lambda, ...
-                       obj, globals);
+                       obj, c, globals);
+
+    delx = x_new - x;
+
+    ## Check if step size has become too small (indicates lack of progress).
+    if (norm (delx) < tol * norm (x))
+      info = 104;
+      break;
+    endif
 
     ## Evaluate objective function, constraints, and gradients at x_new.
-    c_new = feval (obj_grd, x_new);
+    if (have_grd)
+      c_new = feval (obj_grd, x_new);
+    else
+      c_new = feval (obj_grd, x_new, obj_new);
+    endif
 
     ce_new = feval (ce_fun, x_new);
     F_new = feval (ce_grd, x_new);
@@ -458,14 +483,6 @@ function [x, obj, info, iter, nf, lambda] = sqp (x0, objf, cef, cif, lb, ub, max
       y -= t;
     endif
 
-    delx = x_new - x;
-
-    ## Check if step size has become too small (indicates lack of progress).
-    if (norm (delx) < tol * norm (x))
-      info = 104;
-      break;
-    endif
-
     if (have_hess)
 
       B = feval (obj_hess, x);
@@ -556,7 +573,7 @@ endfunction
 
 
 function [x_new, alpha, obj, globals] = ...
-   linesearch_L1 (x, p, obj_fun, obj_grd, ce_fun, ci_fun, lambda, obj, globals)
+   linesearch_L1 (x, p, obj_fun, obj_grd, ce_fun, ci_fun, lambda, obj, c, globals)
 
   ## Choose parameters
   ##
@@ -576,7 +593,6 @@ function [x_new, alpha, obj, globals] = ...
 
   alpha = 1;
 
-  c = feval (obj_grd, x);
   ce = feval (ce_fun, x);
 
   [phi_x_mu, obj, globals] = phi_L1 (obj, obj_fun, ce_fun, ci_fun, x, ...
@@ -611,10 +627,9 @@ function [x_new, alpha, obj, globals] = ...
 endfunction
 
 
-function grd = fdgrd (f, x)
+function grd = fdgrd (f, x, y0)
 
   if (! isempty (f))
-    y0 = feval (f, x);
     nx = length (x);
     grd = zeros (nx, 1);
     deltax = sqrt (eps);
@@ -653,9 +668,9 @@ function jac = fdjac (f, x)
 endfunction
 
 
-function grd = fd_obj_grd (x, obj_fun)
+function grd = fd_obj_grd (x, obj, obj_fun)
 
-  grd = fdgrd (obj_fun, x);
+  grd = fdgrd (obj_fun, x, obj);
 
 endfunction
 
@@ -777,4 +792,3 @@ endfunction
 %!error sqp (1, cell (3,1), cell (2,1), cell (2,1),[],[],1.5)
 %!error sqp (1, cell (3,1), cell (2,1), cell (2,1),[],[],[], ones (2,2))
 %!error sqp (1, cell (3,1), cell (2,1), cell (2,1),[],[],[],-1)
-
