@@ -1,4 +1,9 @@
-## Copyright (C) 1994-2018 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1994-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} axis ()
@@ -32,9 +39,9 @@
 ## first and second elements specify the lower and upper limits for the
 ## x-axis.  The third and fourth specify the limits for the y-axis, the fifth
 ## and sixth specify the limits for the z-axis, and the seventh and eighth
-## specify the limits for the color axis.  The special values -Inf and Inf may
-## be used to indicate that the limit should be automatically computed based
-## on the data in the axes.
+## specify the limits for the color axis.  The special values @code{-Inf} and
+## @code{Inf} may be used to indicate that the limit should be automatically
+## computed based on the data in the axes.
 ##
 ## Without any arguments, @code{axis} turns autoscaling on.
 ##
@@ -63,8 +70,9 @@
 ## @table @asis
 ## @item  @qcode{"auto"}
 ## @itemx @qcode{"auto[xyz]"}
-## Set the specified axes to have nice limits around the data or all if no
-## axes are specified.
+## @itemx @qcode{"auto [xyz]"}
+## Set nice auto-computed limits around the data for all axes, or only
+## the specified axes.
 ##
 ## @item @qcode{"manual"}
 ## Fix the current axes limits.
@@ -83,11 +91,15 @@
 ## The following options affect the appearance of tick marks.
 ##
 ## @table @asis
-## @item @qcode{"tic[xyz]"}
+## @item  @qcode{"tic"}
+## @itemx @qcode{"tic[xyz]"}
+## @itemx @qcode{"tic [xyz]"}
 ## Turn tick marks on for all axes, or turn them on for the specified axes and
 ## off for the remainder.
 ##
-## @item @qcode{"label[xyz]"}
+## @item  @qcode{"label"}
+## @itemx @qcode{"label[xyz]"}
+## @itemx @qcode{"label [xyz]"}
 ## Turn tick labels on for all axes, or turn them on for the specified axes
 ## and off for the remainder.
 ##
@@ -138,8 +150,6 @@
 ## @seealso{xlim, ylim, zlim, caxis, daspect, pbaspect, box, grid}
 ## @end deftypefn
 
-## Author: jwe
-
 function limits = axis (varargin)
 
   [hax, varargin, nargin] = __plt_get_axis_arg__ ("axis", varargin{:});
@@ -205,32 +215,41 @@ function limits = __axis__ (ca, varargin)
         set (ca, "dataaspectratiomode", "auto",
                  "plotboxaspectratio", [1, 1, 1]);
       elseif (strcmp (opt, "equal"))
-        ## Get position of axis in pixels
-        ca_units = get (ca, "units");
-        set (ca, "units", "pixels");
-        axis_pos = get (ca, "position");
-        set (ca, "units", ca_units);
+        is2dview = (get (ca, "view")(2) == 90);
+        if (is2dview)
+          ## Save & later restore axes aspect ratio
+          ca_units = get (ca, "units");
+          set (ca, "units", "pixels");
+          axis_pos = get (ca, "position");
+          set (ca, "units", ca_units);
+          pbratio = [axis_pos(3), axis_pos(4), axis_pos(4)];
+        endif
 
-        pbar = get (ca, "plotboxaspectratio");
-        dx = diff (__get_tight_lims__ (ca, "x"));
-        dy = diff (__get_tight_lims__ (ca, "y"));
-        dz = diff (__get_tight_lims__ (ca, "z"));
-        new_pbar = [dx dy dz];
-        new_pbar(new_pbar == 0) = 1;
-        if (dx/pbar(1) < dy/pbar(2))
-          set (ca, "xlimmode", "auto");
-          new_pbar(1) = (dy / axis_pos(4)) * axis_pos(3);
+        daratio = get (ca, "dataaspectratio");
+        if (all (daratio != [1, 1, 1]))
+          set (ca, "dataaspectratio", [1, 1, 1]);
         else
-          set (ca, "ylimmode", "auto");
-          new_pbar(2) = (dx / axis_pos(3)) * axis_pos(4);
+          set (ca, "dataaspectratio", [1+eps, 1, 1],
+                   "dataaspectratio", [1, 1, 1]);
+        endif
+
+        ## Matlab only adjusts plotboxaspectratio for 2-D figures, but
+        ## Octave needs to do it for 3-D as well.
+        if (is2dview)
+          set (ca, "plotboxaspectratio", pbratio);
+        else
+          pbratio = get (ca, "plotboxaspectratio");
+          if (all (pbratio != [1, 1, 1]))
+            set (ca, "plotboxaspectratio", [1, 1, 1]);
+          else
+            set (ca, "plotboxaspectratio", [1+eps, 1, 1],
+                     "plotboxaspectratio", [1, 1, 1]);
+          endif
         endif
-        set (ca, "dataaspectratio", [1, 1, 1],
-                 "plotboxaspectratio", new_pbar);
       elseif (strcmpi (opt, "vis3d"))
         ## Fix aspect ratio modes for rotation without stretching.
         set (ca, "dataaspectratiomode", "manual",
                  "plotboxaspectratiomode", "manual");
-
       elseif (strcmpi (opt, "normal"))
         ## Set plotboxaspectratio to something obtuse so that switching
         ## back to "auto" will force a re-calculation.
@@ -478,6 +497,8 @@ endfunction
 
 %!demo
 %! clf;
+%! def_ticklen = get (0, "DefaultAxesTickLength");
+%! set (0, "DefaultAxesTickLength", [.028, .025]);
 %! t = 0:0.01:2*pi;
 %! x = sin (t);
 %!
@@ -525,6 +546,8 @@ endfunction
 %!  plot (t, x);
 %!  title ("all ticks and labels");
 %!  axis ("on");
+%!
+%! set (0, "DefaultAxesTickLength", def_ticklen);
 
 %!demo
 %! clf;
