@@ -1,4 +1,9 @@
-## Copyright (C) 2009-2019 Martin Helm
+########################################################################
+##
+## Copyright (C) 2009-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{t}, @var{p}] =} __marching_cube__ (@var{x}, @var{y}, @var{z}, @var{val}, @var{iso})
@@ -25,7 +32,7 @@
 ## @var{iso}.  It is considered that the volume data @var{val} is given at
 ## the points @var{x}, @var{y} and @var{z} which are of type
 ## three-dimensional numeric arrays.  The orientation of the triangles is
-## choosen such that the normals point from the higher values to the lower
+## chosen such that the normals point from the higher values to the lower
 ## values.
 ##
 ## Optionally the color data @var{col} can be passed to this function
@@ -75,8 +82,6 @@
 ##
 ## @end deftypefn
 
-## Author: Martin Helm <martin@mhelm.de>
-
 function [T, p, col] = __marching_cube__ (xx, yy, zz, c, iso, colors)
 
   persistent edge_table = [];
