@@ -1,4 +1,9 @@
-## Copyright (C) 2006-2018 Alexander Barth
+########################################################################
+##
+## Copyright (C) 2006-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{y1}, @var{y2}, @dots{}, @var{y}n] =} ndgrid (@var{x1}, @var{x2}, @dots{}, @var{x}n)
@@ -37,8 +44,6 @@
 ## @seealso{meshgrid}
 ## @end deftypefn
 
-## Author: Alexander Barth <abarth@marine.usf.edu>
-
 function varargout = ndgrid (varargin)
 
   if (nargin == 0)
