@@ -1,6 +1,9 @@
-## Copyright (C) 2016-2018 Cristiano Dorigo, Octave Arena
-## Copyright (C) 2008-2018 Radek Salac
-## Copyright (C) 2012 Carlo de Falco
+########################################################################
+##
+## Copyright (C) 2008-2020 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -17,12 +20,14 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{x} =} bicgstab (@var{A}, @var{b}, @var{tol}, @var{maxit}, @var{M1}, @var{M2}, @var{x0}, @dots{})
 ## @deftypefnx {} {@var{x} =} bicgstab (@var{A}, @var{b}, @var{tol}, @var{maxit}, @var{M}, [], @var{x0}, @dots{})
 ## @deftypefnx {} {[@var{x}, @var{flag}, @var{relres}, @var{iter}, @var{resvec}] =} bicgstab (@var{A}, @var{b}, @dots{})
-## Solve @code{A x = b} using the stabilizied Bi-conjugate gradient iterative
+## Solve @code{A x = b} using the stabilized Bi-conjugate gradient iterative
 ## method.
 ##
 ## The input parameters are:
@@ -90,7 +95,7 @@
 ## @item @var{relres} is the relative residual obtained with as
 ## @code{(@var{A}*@var{x}-@var{b}) / @code{norm(@var{b})}}.
 ##
-## @item @var{iter} is the (possibily half) iteration which @var{x} is
+## @item @var{iter} is the (possibly half) iteration which @var{x} is
 ## computed.  If it is an half iteration then it is @code{@var{iter} + 0.5}
 ##
 ## @item @var{resvec} is a vector containing the residual of each half and
@@ -192,15 +197,11 @@
 ## @end group
 ## @end example
 ##
-## References:
+## Reference:
 ##
-## @enumerate
-##
-## @item @nospell{Y. Saad}, @cite{Iterative Methods for Sparse Linear
+## @nospell{Y. Saad}, @cite{Iterative Methods for Sparse Linear
 ## Systems}, Second edition, 2003, SIAM
 ##
-## @end enumerate
-##
 ## @seealso{bicg, cgs, gmres, pcg, qmr, tfqmr}
 ##
 ## @end deftypefn
@@ -236,7 +237,7 @@ function [x_min, flag, relres, iter_min, resvec] = ...
   resvec = zeros (d_maxit,1);
   x = x_min = x_pr = x0;
   iter = iter_min = 0;
-  ## deafult setting of flag is 1 (i.e. max number of iterations reached)
+  ## default setting of flag is 1 (i.e. max number of iterations reached)
   flag = 1;
 
   res = b - feval (Afun, x, varargin{:});
